;===================================================================;
;                                                                   ;
; Hangman                                                           ;
; Main Game Loop                                                    ;
;                                                                   ;
;===================================================================;

iniQuestion:
        xor     a
        ld      (charsLeft),a
        ld      (charsWrong),a
        ld      hl,lettersUsed
        ld      b,26
resetLettersUsed:
        ld      (hl),a                          ; Reset the lettersUsed array
        inc     hl
        djnz    resetLettersUsed

        ld      hl,gamepicData                  ; HL => Start of compressed data
        ld      de,videoBuffer                  ; DE => Destination for uncompressed data
        ld      ix,gbuf                         ; IX => 1024 bytes free mem to be used by huffExtr
        call    huffExtr                        ; Extract game background to gbuf

        ld      hl,videoBuffer
        ld      de,gbuf
#ifndef TI86
        ld      bc,768
#else
        ld      bc,1024
#endif
        ldir                                    ; Load uncompressed data to gbuf

        ld      hl,(libNamePtr)
        ld      a,(question)
        ld      b,a
        inc     b                               ; B = Category number + 1
findCatName:
        call    nextString
        djnz    findCatName                     ; Search through strings to find category name
#ifndef TI86
        ld      de,26*256+3
#else
        ld      de,26*256+19
#endif
        call    vPuts                           ; Write category name to gbuf

        ld      a,5
        ld      (putSpriteHeight),a             ; Set height of sprites to 5
        ld      b,56                            ; Number of letters to write/draw
        ld      de,question+1                   ; DE => Letter data
writeQuestionLoop:
        push    bc
        push    de
        ld      a,(de)
        ld      de,sprBlack
        cp      ' '
        jr      z,writeQuestionShowLetter
        ld      de,sprHyphen
        cp      '-'
        jr      z,writeQuestionShowLetter
        ld      hl,charsLeft
        inc     (hl)
        ld      de,sprGray
writeQuestionShowLetter:
        ld      a,b                             ; A = Counter
        sub     56
        neg                                     ; A = What number letter we're up to
        add     a,a                             ; Double to get offset in questionLetterCoords table
        ld      c,a
        ld      b,0
        ld      hl,questionLetterCoords
        add     hl,bc                           ; HL => Coords to use for putSprite
        bcall(_ldhlind)                         ; H = X coord, Y = Y coord
        ld      a,h
        call    putSprite                       ; Draw/write letter
        pop     de
        pop     bc
        inc     de                              ; DE => Next letter data
        djnz    writeQuestionLoop

        bcall(GET_KEY)                          ; Clear key input buffer

;================================================
; MAIN LOOP START
;================================================
mainLoop:
        xor     a
        ld      (charsRevealed),a               ; Reset counter
        ld      a,5
        ld      (putSpriteHeight),a             ; Set height of sprites to 5

        call    ionFastCopy

        call    waitKey
#ifndef TI86
        cp      GK_MODE
#else
        cp      GK_EXIT
#endif
        jp      z,quit                          ; If [MODE]/[EXIT] pressed, quit

        call    getLetter
        or      a                               ; Does A = 0?
        jr      z,mainLoop                      ; If no valid key pressed, return to start of loop
        sub     'A'
        ld      e,a
        ld      d,0
        ld      hl,lettersUsed
        add     hl,de                           ; HL => Flag for letter pressed
        ld      a,(hl)
        or      a
        jr      nz,mainLoop                     ; If it's been used, return to start of loop

        inc     (hl)                            ; Otherwise, flag it as having been used
        ld      a,e
        push    af
        add     a,a                             ; A = Offset in usedLetterCoords
        ld      e,a                             ; DE = Offset in usedLetterCoords (D = 0 still from earlier)
        ld      hl,usedLetterCoords
        add     hl,de                           ; HL => Coords to use
        bcall(_ldhlind)                         ; H = X coord, L = Y coord
        ld      a,h                             ; A = X coord, L = Y coord
        ld      de,sprBlack
        call    putSprite                       ; Put black block over letter to show it's been used
        pop     af
        add     a,'A'                           ; A = Letter selected
        ld      hl,question+1                   ; HL => Start of letter data for question
        ld      b,56                            ; Number of letters to check

checkNewLetterLoop:                             ; Loop to check if letter reveals anything
        push    af
        push    bc
        push    hl
        cp      (hl)                            ; Check letter selected against letter in question
        ex      de,hl                           ; DE => Current letter in question data
        jr      nz,endCheckNewLetterLoop
        push    af
        ld      a,b                             ; A = Counter
        sub     56
        neg                                     ; A = What number letter we're up to
        add     a,a                             ; Double to get offset in questionLetterCoords table
        ld      c,a
        ld      b,0
        ld      hl,questionLetterCoords
        add     hl,bc                           ; HL => Coords to use for putSprite
        bcall(_ldhlind)                         ; H = X coord, Y = Y coord

        push    hl
        push    hl
        pop     bc                              ; B = X coord, C = Y coord
        ld      hl,sprBlank
        call    putClippedMaskedSprite          ; Blank out letter space so that we can draw/write the letter
        pop     de                              ; D = X coord, E = Y coord
        pop     af
        sub     'A'
        ld      l,a                             ; H = Letter pressed
        ld      h,5
        push    de
        bcall(_htimesl)                         ; L = Sprite offset
        pop     de                              ; D = X coord, E = Y coord
        ld      bc,sprA
        add     hl,bc                           ; HL => Letter sprite
        ex      de,hl                           ; HL = Coords, DE => Letter sprite
        ld      a,h
        call    putSprite                       ; Draw/write letter
        ld      hl,charsRevealed
        inc     (hl)                            ; Increase counter
        ld      hl,charsLeft
        dec     (hl)                            ; Check if 0 chars left
        jp      z,wonGame                       ; If so, the game is won!
endCheckNewLetterLoop:
        pop     hl
        pop     bc
        inc     hl                              ; HL => Next letter data
        pop     af
        djnz    checkNewLetterLoop

        ld      a,(charsRevealed)
        or      a
        jp      nz,mainLoop
        ld      a,8
        ld      (putSpriteHeight),a             ; Set sprite height back to 8
        ld      hl,charsWrong
        inc     (hl)
        ld      a,(hl)
        dec     a                               ; A = Number of wrong letters - 1
        add     a,a
        add     a,a
        add     a,a                             ; x8 for offset in hangTable
        ld      l,a
        ld      h,0
        ld      de,hangTable
        add     hl,de                           ; HL => Entry in hangTable for data required
        ld      b,2                             ; B = 2 sprites to draw

drawHangLoop:
        push    bc
        push    hl
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE => Sprite
        inc     hl
        bcall(_ldhlind)                         ; H = X coord, L = Y coord
        ld      a,h                             ; A = X coord, L = Y coord
        call    putSprite
        pop     hl
        ld      de,4
        add     hl,de                           ; HL => Next entry in hangTable
        pop     bc
        djnz    drawHangLoop

        ld      a,(charsWrong)
        cp      5
        jp      c,mainLoop
        jp      lostGame

.end
